@@ -253,26 +253,23 @@ class Intersection(Operation):
 class Union(Operation):
     __slots__ = ()
 
-    def compute(self) -> Union_[
-        Empty, Mix, Multipolygon, Multisegment, Polygon,
-        Segment]:
+    def compute(self) -> Union_[Empty, Mix, Multipolygon, Multisegment,
+                                Polygon, Segment]:
         context = self.context
         linear_box, shaped_box = (context.segments_box(self.linear.segments),
                                   context.polygons_box(self.shaped.polygons))
         if bounding.disjoint_with(linear_box, shaped_box):
             return context.mix_cls(context.empty, self.linear.value,
                                    self.shaped.value)
-        self.linear.segments = bounding.to_intersecting_segments(
-                shaped_box, self.linear.segments, context)
-        if not self.linear.segments:
-            return context.empty
+        result_segments, self.linear.segments = (
+            bounding.split_intersecting_segments(
+                    shaped_box, self.linear.segments, context))
         self.shaped.polygons = bounding.to_intersecting_polygons(
                 linear_box, self.shaped.polygons, context)
-        if not self.shaped.polygons:
-            return context.empty
         segments = endpoints_to_segments([to_endpoints(event)
                                           for event in self.sweep()
                                           if event.in_result], context)
+        segments.extend(result_segments)
         linear = unpack_segments(segments, context)
         return (self.shaped.value
                 if linear is context.empty
